library ieee;
  use ieee.std_logic_1164.all;
  use ieee.std_logic_arith.all;
  use ieee.std_logic_unsigned.all;

entity blink is
  port (
    clk1    : in  std_logic;
    clk2    : in  std_logic;
    reset_l : in  std_logic;
    led1    : out std_logic;
    led2    : out std_logic
  );
end blink;

architecture behav of blink is

signal led1_27    : std_logic_vector(26 downto 0);
signal led2_27    : std_logic_vector(26 downto 0);

begin

-- * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
-- led1 is driven by a 27-bit counter clocked by the rad system clock (100mhz)
-- led2 is driven by a 27-bit counter clocked by the rad system clock b (100mhz)
-- * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

-- a flip-flop is added after the last counter stage so that it can 
-- sit in the iob.

-- led1 logic
led1_proc : process (clk1, reset_l)
  begin
    if (reset_l = '0') then
      led1 <=  '1';
    elsif (clk1'event and clk1 = '1') then
      led1 <= led1_27(26);
    end if;
  end process;
  

  
-- divide by 67 million
led1_cntr_proc : process (clk1, reset_l)
  begin
    if (reset_l = '0') then
      led1_27 <= (others => '0');
    elsif (clk1'event and clk1 = '1') then
      led1_27 <= unsigned (led1_27) + 1;
    end if;
  end process;
  
-- * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
-- led2 logic
led2_proc : process (clk2, reset_l)
  begin
    if (reset_l = '0') then
      led2 <=  '1';
    elsif (clk2'event and clk2 = '1') then
      led2 <= led2_27(26);
    end if;
  end process;
  

  
-- divide by 67 million
led2_cntr_proc : process (clk2, reset_l)
  begin
    if (reset_l = '0') then
      led2_27 <= (others => '0');
    elsif (clk2'event and clk2 = '1') then
      led2_27 <= unsigned (led2_27) + 1;
    end if;
  end process;

end behav;
