--  applied research laboratory
--  washington university in st. louis
--
--  file: loopback_module.vhd
--  entity declaration for rad module
--  created by:  david e. taylor (det3@arl.wustl.edu)
--  created on:  august, 16 2000
--  last modified:  august 18, 2000 @ 11:20 am
--
--  important:  refer to rad module interface specification
--  for explanations and timing specifications for all interface
--  signals.

library ieee;
use ieee.std_logic_1164.all;

entity loopback_module is
  port (
    -- clock & reset
    clk     : in std_logic;             -- 100mhz global clock
    reset_l : in std_logic;             -- synchronous reset, asserted-low

    -- cell input interface
    soc_mod_in   : in  std_logic;                      -- start of cell
    d_mod_in     : in  std_logic_vector(31 downto 0);  -- 32-bit data
    tca_mod_in   : out std_logic;                      -- transmit cell available

    -- cell output interface
    soc_out_mod   : out std_logic;                      -- start of cell
    d_out_mod     : out std_logic_vector(31 downto 0);  -- 32-bit data
    tca_out_mod   : in  std_logic                       -- transmit cell available
    );
end loopback_module;

architecture behavioral of loopback_module is

  signal soc : std_logic;               -- start of cell
  signal data : std_logic_vector(31 downto 0);  -- 32-bit words of atm cell
  signal tca : std_logic;               -- transmit cell available

begin  -- behavioral

-- pass cells through.  on reconfig, hold tca low.

  cell_io_ff : process (clk)
  begin  -- process cell_ff
    if (clk='1' and clk'event) then
      if reset_l='0' then
        soc <= '0';
        soc_out_mod <= '0';
        data <= (others=>'0') ;
        d_out_mod <= (others=>'0') ;
        tca <= '0' ;
        tca_mod_in <= '0' ;
      else
        soc <= soc_mod_in ;
        data <= d_mod_in ;
        soc_out_mod <= soc ;
        d_out_mod <= data ;
        tca <= tca_out_mod ;
        tca_mod_in <= tca ;
      end if;
    end if;
  end process cell_io_ff;
  
end behavioral;
