--  applied research laboratory
--  washington university in st. louis
--
--  file:  rad_loopback_core.vhd
--  top level structure for rad fpga with ingress/egress loopback modules
--  created by:  john w. lockwood (lockwood@arl.wustl.edu),
--  david e. taylor (det3@arl.wustl.edu)
--
library ieee;
use ieee.std_logic_1164.all;

entity rad_loopback_core is
  port (
    -- clocks
    rad_clk  : in std_logic;
    rad_clkb : in std_logic;

    -- reset & reconfig
    rad_reset : in std_logic;
    rad_ready : out std_logic;

    -- ingress path
    -- input
    soc_lc_nid   : in  std_logic;
    d_lc_nid     : in  std_logic_vector(31 downto 0);
    tcaff_lc_rad : out std_logic;
    -- output
    soc_lc_rad   : out std_logic;
    d_lc_rad     : out std_logic_vector(31 downto 0);
    tcaff_lc_nid : in  std_logic;

    -- egress path
    -- input
    soc_sw_nid   : in  std_logic;
    d_sw_nid     : in  std_logic_vector(31 downto 0);
    tcaff_sw_rad : out std_logic;
    -- output
    soc_sw_rad   : out std_logic;
    d_sw_rad     : out std_logic_vector(31 downto 0);
    tcaff_sw_nid : in  std_logic;

    -- test connector pins
--    rad_test1 : out std_logic_vector(15 downto 0);
--    rad_test2 : out std_logic_vector(15 downto 0);

    -- test led pins
    rad_led1 : out std_logic;
    rad_led2 : out std_logic;
    rad_led3 : out std_logic;
    rad_led4 : out std_logic
    );
end rad_loopback_core;
-------------------------------------------------------------------------------
architecture structure of rad_loopback_core is

-- component declarations

  component loopback_module
    port (
      clk         : in  std_logic;
      reset_l     : in  std_logic;
      soc_mod_in  : in  std_logic;
      d_mod_in    : in  std_logic_vector(31 downto 0);
      tca_mod_in  : out std_logic;
      soc_out_mod : out std_logic;
      d_out_mod   : out std_logic_vector(31 downto 0);
      tca_out_mod : in  std_logic
      );
  end component;

  component app_module
    port (
      clk         : in  std_logic;
      reset_l     : in  std_logic;
      soc_mod_in  : in  std_logic;
      d_mod_in    : in  std_logic_vector(31 downto 0);
      tca_mod_in  : out std_logic;
      soc_out_mod : out std_logic;
      d_out_mod   : out std_logic_vector(31 downto 0);
      tca_out_mod : in  std_logic;
      enable_l    : in  std_logic;
      ready_l     : out std_logic
      );
  end component;

  component blink
    port (
      clk1    : in  std_logic;
      clk2    : in  std_logic;
      reset_l : in  std_logic;
      led1    : out std_logic;
      led2    : out std_logic);
  end component;

-- signal declarations

--  signal ingress_test, egress_test : std_logic_vector(31 downto 0);
  signal logic0, logic1 : std_logic;

begin  -- structural

  rad_ready <= not(rad_reset);

-- test pin flops
  test_pin_ff : process (rad_clk)
  begin  -- process test_pin_ff
    if rad_clk'event and rad_clk = '1' then  -- rising clock edge
--        rad_test2 <= ingress_test(31 downto 16);
--        rad_test1 <= ingress_test(15 downto 0);

        rad_led3 <= rad_reset;
        rad_led4 <= not rad_reset;
    end if;
  end process test_pin_ff;

  logic0 <= '0';
  logic1 <= '1';


  ingress : app_module 
    port map (
      clk         => rad_clk,
      reset_l     => rad_reset,
      soc_mod_in  => soc_lc_nid,
      d_mod_in    => d_lc_nid,
      tca_mod_in  => tcaff_lc_rad,
      soc_out_mod => soc_lc_rad,
      d_out_mod   => d_lc_rad,
      tca_out_mod => tcaff_lc_nid,
      enable_l    => logic0,
      ready_l     => open);

  egress : loopback_module
    port map (
      clk         => rad_clkb,
      reset_l     => rad_reset,
      soc_mod_in  => soc_sw_nid,
      d_mod_in    => d_sw_nid,
      tca_mod_in  => tcaff_sw_rad,
      soc_out_mod => soc_sw_rad,
      d_out_mod   => d_sw_rad,
      tca_out_mod => tcaff_sw_nid);

  blink1 : blink
    port map (
      clk1    => rad_clk,
      clk2    => rad_clkb,
      reset_l => rad_reset,
      led1    => rad_led1,
      led2    => rad_led2);

end structure;
